/* Sliderbuttons.ts

	Purpose:

	Description:

	History:
		Thu Sep 12 11:19:05 CST 2019, Created by jameschu

Copyright (C) 2019 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
(() => {
/**
 * A Sliderbuttons.
 *
 * @author jameschu
 * @since 9.0.0
 */
zkex.slider.Sliderbuttons = zk.$extends(zul.Widget, {
    _values: null,
    _dragStartBtn: null,
    _dragEndBtn: null,

    $init() {
        this.$supers('$init', arguments);
        this._values = [0, 0];
    },

    $define: {
        /**
         * Returns the values of this sliderbuttons.
         * <p>Default: [0, 0]
         * @return int[]
         */
        /**
         * Sets the values of this sliderbuttons.
         * @param int[] values
         */
        values(v: number[]) {
            if (this.desktop) {
                const p = this.parent;
                if (p)
                    p.updatePosByValues_(this);
            }
        }
    },

    bind_() {
        this.$supers(zkex.slider.Sliderbuttons, 'bind_', arguments);
        const p = this.parent;
        if (p) {
            const $clz = this.$class,
                opt = {
                    constraint: p._orient,
                    snap: $clz._snap,
                    change: $clz._dragging,
                    ignoredrag: $clz._ignoredrag,
                    endeffect: $clz._endDrag
                };

            if (p.isTooltipVisible())
                this.setTooltipVisible(true);

            this._dragStartBtn = new zk.Draggable(this, this.$n('start-btn'), opt);
            this._dragEndBtn = new zk.Draggable(this, this.$n('end-btn'), opt);
        }
    },
    unbind_() {
        if (this._dragEndBtn) {
            this._dragEndBtn.destroy();
            this._dragEndBtn = null;
        }
        if (this._dragStartBtn) {
            this._dragStartBtn.destroy();
            this._dragStartBtn = null;
        }
        this.$supers(zkex.slider.Sliderbuttons, 'unbind_', arguments);
    },

    setTooltipVisible(visible: boolean) {
        if (this.desktop) {
            const nStartTooltip = this.$n('start-tooltip'),
                nEndTooltip = this.$n('end-tooltip');
            if (visible) {
                nStartTooltip.style.display = 'block';
                nEndTooltip.style.display = 'block';
            } else {
                nStartTooltip.style.display = '';
                nEndTooltip.style.display = '';
            }
        }
    },

    doClick_(evt) {
        const jqTarget = jq(evt.domTarget);
        if (jqTarget.hasClass(this.$s('button')))
            jqTarget.focus();
        this.$supers('doClick_', arguments);
    },

    syncArea() {
        const p = this.parent;
        if (!p) return;
        const isVert = p.isVertical(),
            ofDim = isVert ? 'top' : 'left',
            startBtn = this.$n('start-btn'),
            arean = this.$n('area'),
            startBtnOffsetP = startBtn.style[ofDim] || '0';
        arean.style[ofDim] = startBtnOffsetP;
        arean.style[isVert ? 'height' : 'width'] = (parseFloat(this.$n('end-btn').style[ofDim] || '0') - parseFloat(startBtnOffsetP)) + '%';
        p.updateActiveMarks_();

        // update tooltip value
        const vals = this._values;
        jq(this.$n('start-tooltip')).html(vals[0]);
        jq(this.$n('end-tooltip')).html(vals[1]);
    },

    _realSlidePos(btn: HTMLButtonElement, pos: number[], showTooltip: boolean) {
        const p = this.parent;
        if (!p) return;
        let isVert = p.isVertical(),
            dimStr = isVert ? 'top' : 'left',
            sliderMin = p.getMin(),
            sliderMax = p.getMax(),
            totalDim = p.getSliderSize_(),
            btnOffset = parseInt(btn.style[dimStr]) / 100 * totalDim,
            firstBtn = this.$n('start-btn'),
            isFirstBtn = btn == firstBtn,
            minOffset = 0,
            minVal = sliderMin,
            maxOffset = totalDim,
            maxVal = sliderMax,
            dimIndex = isVert ? 1 : 0,
            newSlidePos = pos[dimIndex],
            valueIndex = isFirstBtn ? 0 : 1,
            newVal;

        if (newSlidePos <= minOffset) {
            newSlidePos = minOffset;
            newVal = minVal;
        } else if (newSlidePos >= maxOffset) {
            newSlidePos = maxOffset;
            newVal = maxVal;
        } else {
            let slidePos = btnOffset,
                step = p.getStep(),
                stepOffset = totalDim / (sliderMax - sliderMin) * step,
                newSlideStepCnt = newSlidePos / stepOffset;

            newSlideStepCnt = newSlidePos - slidePos > 0 ? Math.floor(newSlideStepCnt) : Math.ceil(newSlideStepCnt);
            newSlidePos = Math.round(stepOffset * newSlideStepCnt);
            if (newSlidePos <= minOffset) {
                newSlidePos = minOffset;
                newVal = minVal;
            } else if (newSlidePos >= maxOffset) {
                newSlidePos = maxOffset;
                newVal = maxVal;
            } else {
                newVal = sliderMin + newSlideStepCnt * step;
            }
        }
        pos[dimIndex] = newSlidePos;
        //update value
        this._values[valueIndex] = newVal;
        const jqBtnTooltip = jq(btn).find('.' + this.$s('tooltip'));
        jqBtnTooltip.html(newVal);
        if (showTooltip)
            jqBtnTooltip.show();
        return pos;
    },

    _afterDragButtons() {
        const p = this.parent;
        if (!p) return;
        if (!p.buttonsFireChangeBlocked_)
            this.fire('onRangeValueChange', {values: this._values});
        p.afterBtnsValuesChange_(this);
    },

    isDisabled() {
        const p = this.parent;
        return !p || p.isDisabled();
    },

    isTooltipVisible() {
        const p = this.parent;
        return !p || p.isTooltipVisible();
    }
}, {
    _ignoredrag(dg) {
        return dg.control.isDisabled();
    },
    _snap(dg, pos) {
        return dg.control._realSlidePos(dg.node, pos, true);
    },
    _dragging(dg) {
        // change px to %
        const btns = dg.control,
            p = btns.parent,
            dimStr = p.isVertical() ? 'top' : 'left',
            nBtn = dg.node,
            nBtnDim = (parseInt(nBtn.style[dimStr]) * 100 / p.getSliderSize_()) + '%';
        nBtn.style[dimStr] = nBtnDim;

        // check other button
        const firstBtn = btns.$n('start-btn');
        if (btns._values[0] > btns._values[1]) {
            if (nBtn == firstBtn) {
                btns._values[1] = btns._values[0];
                btns.$n('end-btn').style[dimStr] = nBtnDim;
            } else {
                btns._values[0] = btns._values[1];
                firstBtn.style[dimStr] = nBtnDim;
            }
        }
        btns.syncArea();
        p.fixOtherChildrenValues_(btns);
    },
    _endDrag(dg) {
        const wgt = dg.control;
        if (!wgt.isTooltipVisible())
            jq(dg.node).find('.' + wgt.$s('tooltip'))[0].style.display = '';
        wgt._afterDragButtons();
    }
});
})();
